/**
 * Copyright (c) 2008-2022, MOVES Institute, Naval Postgraduate School (NPS). All rights reserved.
 * This work is provided under a BSD open-source license, see project license.html and license.txt
 */
// header autogenerated using string template dis7javalicense.txt

package edu.nps.moves.dis7.pdus;

import java.util.*;
import java.io.*;
import edu.nps.moves.dis7.enumerations.*;
import java.nio.ByteBuffer;

/**
 * 5.12.4.14 Used to communicate a request for data in record format.
 * IEEE Std 1278.1-2012, IEEE Standard for Distributed Interactive Simulation - Application Protocols
 */
public class RecordQueryRPdu extends SimulationManagementWithReliabilityFamilyPdu implements Serializable
{
   /** request ID */
   protected int requestID;

   /** level of reliability service used for this transaction uid 74 */
   protected RequiredReliabilityService requiredReliabilityService = RequiredReliabilityService.values()[0];

   /** padding */
   protected byte pad1;

   /** event type uid 334 */
   protected RecordQueryREventType eventType = RecordQueryREventType.values()[0];

   /** time */
   protected int time;

   /** numberOfRecords */
   protected int numberOfRecords;

   /** record IDs */
   protected List< RecordQuerySpecification > recordIDs = new ArrayList< RecordQuerySpecification >();
 

/** Constructor creates and configures a new instance object */
 public RecordQueryRPdu()
 {
    setPduType( DisPduType.RECORD_QUERY_RELIABLE );
 }
/** copy method creates a deep copy of current object using preferred marshalling method
 * @return deep copy of PDU */
 public RecordQueryRPdu copy()
 {
     return copyDataOutputStream();
 }
/** Creates a "deep copy" of current object using ByteBuffer methods.
 * @return deep copy of PDU */
 public RecordQueryRPdu copyByteBuffer()
 {
     RecordQueryRPdu newCopy = new RecordQueryRPdu();
     ByteBuffer byteBuffer = ByteBuffer.allocate(400);
     try
     {
         this.marshal(byteBuffer);      // working
         newCopy.unmarshal(byteBuffer); // not working
     }
     catch (Exception e)
     {
         System.out.println("RecordQueryRPdu deep copy() marshall/unmarshall ByteBuffer exception " + e.getMessage());
         e.printStackTrace();
         System.exit(-1);
     }
     return newCopy;
 }

/** byteArrayOutputStream (baos) is used for marshal/unmarshal serialization
   * @see copyDataOutputStream() */
protected ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
/** dataOutputStream (dos) is used for marshal/unmarshal serialization
   * @see copyDataOutputStream() */
protected DataOutputStream      dataOutputStream      = new DataOutputStream(byteArrayOutputStream);

/** copy method creates a deep copy of current object using DataOutputStream methods.
 * @return deep copy of PDU */
 public RecordQueryRPdu copyDataOutputStream()
 {
     RecordQueryRPdu newCopy = new RecordQueryRPdu();
     try
     {
         this.marshal(dataOutputStream);
         byte[] byteArrayDOS = byteArrayOutputStream.toByteArray();
         newCopy.unmarshal(ByteBuffer.wrap(byteArrayDOS));
     }
     catch (Exception e)
     {
         System.out.println("RecordQueryRPdu deep copy() marshall/unmarshall DataOutputStream exception " + e.getMessage());
         e.printStackTrace();
         System.exit(-1);
     }
        try
        {
                 dataOutputStream.flush();
            byteArrayOutputStream.flush();
            byteArrayOutputStream.reset();
        }
        catch (IOException ioe)
        {
            System.out.println("RecordQueryRPdu copyDataOutputStream() flush IOException: " + ioe.getMessage());
        }
     return newCopy;
 }

  /**
   * Returns size of this serialized (marshalled) object in bytes
   * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
   * @return serialized size in bytes
   */
public int getMarshalledSize()
{
   int marshalSize = 0; 

   marshalSize = super.getMarshalledSize();
   marshalSize += 4;  // requestID
   if (requiredReliabilityService != null)
       marshalSize += requiredReliabilityService.getMarshalledSize();
   marshalSize += 1;  // pad1
   if (eventType != null)
       marshalSize += eventType.getMarshalledSize();
   marshalSize += 4;  // time
   marshalSize += 4;  // numberOfRecords
   if (recordIDs != null)
       for (int idx=0; idx < recordIDs.size(); idx++)
       {
            RecordQuerySpecification listElement = recordIDs.get(idx);
            marshalSize += listElement.getMarshalledSize();
       }

   return marshalSize;
}


/** Setter for {@link RecordQueryRPdu#requestID}
  * @param pRequestID new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setRequestID(int pRequestID)
{
    requestID = pRequestID;
    return this;
}
/** Getter for {@link RecordQueryRPdu#requestID}
  * @return value of interest */
public int getRequestID()
{
    return requestID; 
}

/** Setter for {@link RecordQueryRPdu#requiredReliabilityService}
  * @param pRequiredReliabilityService new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setRequiredReliabilityService(RequiredReliabilityService pRequiredReliabilityService)
{
    requiredReliabilityService = pRequiredReliabilityService;
    return this;
}
/** Getter for {@link RecordQueryRPdu#requiredReliabilityService}
  * @return value of interest */
public RequiredReliabilityService getRequiredReliabilityService()
{
    return requiredReliabilityService; 
}

/** Setter for {@link RecordQueryRPdu#pad1}
  * @param pPad1 new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setPad1(byte pPad1)
{
    pad1 = pPad1;
    return this;
}
/** Utility setter for {@link RecordQueryRPdu#pad1}
  * @param pPad1 new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setPad1(int pPad1){
    pad1 = (byte) pPad1;
    return this;
}
/** Getter for {@link RecordQueryRPdu#pad1}
  * @return value of interest */
public byte getPad1()
{
    return pad1; 
}

/** Setter for {@link RecordQueryRPdu#eventType}
  * @param pEventType new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setEventType(RecordQueryREventType pEventType)
{
    eventType = pEventType;
    return this;
}
/** Getter for {@link RecordQueryRPdu#eventType}
  * @return value of interest */
public RecordQueryREventType getEventType()
{
    return eventType; 
}

/** Setter for {@link RecordQueryRPdu#time}
  * @param pTime new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setTime(int pTime)
{
    time = pTime;
    return this;
}
/** Getter for {@link RecordQueryRPdu#time}
  * @return value of interest */
public int getTime()
{
    return time; 
}

/** Setter for {@link RecordQueryRPdu#recordIDs}
  * @param pRecordIDs new value of interest
  * @return same object to permit progressive setters */
public RecordQueryRPdu setRecordIDs(List<RecordQuerySpecification> pRecordIDs)
{
    recordIDs = pRecordIDs;
    return this;
}
/** Getter for {@link RecordQueryRPdu#recordIDs}
  * @return value of interest */
public List<RecordQuerySpecification> getRecordIDs()
{
    return recordIDs; 
}

/**
 * Serializes an object to a DataOutputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataOutputStream
 * @param dos the OutputStream
 */
public void marshal(DataOutputStream dos) throws Exception
{
    super.marshal(dos);
    try 
    {
       dos.writeInt(requestID);
       requiredReliabilityService.marshal(dos);
       dos.writeByte(pad1);
       eventType.marshal(dos);
       dos.writeInt(time);
       dos.writeInt(recordIDs.size());

       for (int idx = 0; idx < recordIDs.size(); idx++)
       {
            RecordQuerySpecification aRecordQuerySpecification = recordIDs.get(idx);
            aRecordQuerySpecification.marshal(dos);
       }

    }
    catch(Exception e)
    {
      System.err.println(e);
    }
}

/**
 * Deserializes an object from a DataInputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataInputStream
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param dis the InputStream
 * @return marshalled serialized size in bytes
 */
public int unmarshal(DataInputStream dis) throws Exception
{
    int uPosition = 0;
    uPosition += super.unmarshal(dis);

    try 
    {
        requestID = dis.readInt();
        uPosition += 4;
        requiredReliabilityService = RequiredReliabilityService.unmarshalEnum(dis);
        uPosition += requiredReliabilityService.getMarshalledSize();
        pad1 = (byte)dis.readUnsignedByte();
        uPosition += 1;
        eventType = RecordQueryREventType.unmarshalEnum(dis);
        uPosition += eventType.getMarshalledSize();
        time = dis.readInt();
        uPosition += 4;
        numberOfRecords = dis.readInt();
        uPosition += 4;
        for (int idx = 0; idx < numberOfRecords; idx++)
        {
            RecordQuerySpecification anX = new RecordQuerySpecification();
            uPosition += anX.unmarshal(dis);
            recordIDs.add(anX);
        }

    }
    catch(Exception e)
    { 
      System.err.println(e); 
    }
    return getMarshalledSize();
}

/**
 * Packs an object into the ByteBuffer.
 * @throws java.nio.BufferOverflowException if byteBuffer is too small
 * @throws java.nio.ReadOnlyBufferException if byteBuffer is read only
 * @see java.nio.ByteBuffer
 * @param byteBuffer The ByteBuffer at the position to begin writing
 * @throws Exception ByteBuffer-generated exception
 */
public void marshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
   super.marshal(byteBuffer);
   byteBuffer.putInt( (int)requestID);
   requiredReliabilityService.marshal(byteBuffer);
   byteBuffer.put( (byte)pad1);
   eventType.marshal(byteBuffer);
   byteBuffer.putInt( (int)time);
   byteBuffer.putInt( (int)recordIDs.size());

   for (int idx = 0; idx < recordIDs.size(); idx++)
   {
        RecordQuerySpecification aRecordQuerySpecification = recordIDs.get(idx);
        aRecordQuerySpecification.marshal(byteBuffer);
   }

}

/**
 * Unpacks a Pdu from the underlying data.
 * @throws java.nio.BufferUnderflowException if byteBuffer is too small
 * @see java.nio.ByteBuffer
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param byteBuffer The ByteBuffer at the position to begin reading
 * @return marshalled serialized size in bytes
 * @throws Exception ByteBuffer-generated exception
 */
public int unmarshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
    super.unmarshal(byteBuffer);

    try
    {
        // attribute requestID marked as not serialized
        requestID = byteBuffer.getInt();
        // attribute requiredReliabilityService marked as not serialized
        requiredReliabilityService = RequiredReliabilityService.unmarshalEnum(byteBuffer);
        // attribute pad1 marked as not serialized
        pad1 = (byte)(byteBuffer.get() & 0xFF);
        // attribute eventType marked as not serialized
        eventType = RecordQueryREventType.unmarshalEnum(byteBuffer);
        // attribute time marked as not serialized
        time = byteBuffer.getInt();
        // attribute numberOfRecords marked as not serialized
        numberOfRecords = byteBuffer.getInt();
        // attribute recordIDs marked as not serialized
        for (int idx = 0; idx < numberOfRecords; idx++)
        {
        RecordQuerySpecification anX = new RecordQuerySpecification();
        anX.unmarshal(byteBuffer);
        recordIDs.add(anX);
        }

    }
    catch (java.nio.BufferUnderflowException bue)
    {
        System.err.println("*** buffer underflow error while unmarshalling " + this.getClass().getName());
    }
    return getMarshalledSize();
}

 /*
  * Override of default equals method.  Calls equalsImpl() for content comparison.
  */
@Override
 public boolean equals(Object obj)
 {
    if(this == obj)
      return true;

    if(obj == null)
       return false;

    if(!getClass().isAssignableFrom(obj.getClass())) //if(getClass() != obj.getClass())
        return false;

    return equalsImpl(obj);
 }

@Override
 public boolean equalsImpl(Object obj)
 {
     boolean ivarsEqual = true;

     final RecordQueryRPdu rhs = (RecordQueryRPdu)obj;

     if( ! (requestID == rhs.requestID)) ivarsEqual = false;
     if( ! (requiredReliabilityService == rhs.requiredReliabilityService)) ivarsEqual = false;
     if( ! (pad1 == rhs.pad1)) ivarsEqual = false;
     if( ! (eventType == rhs.eventType)) ivarsEqual = false;
     if( ! (time == rhs.time)) ivarsEqual = false;

     for (int idx = 0; idx < recordIDs.size(); idx++)
        if( ! ( recordIDs.get(idx).equals(rhs.recordIDs.get(idx)))) ivarsEqual = false;

    return ivarsEqual && super.equalsImpl(rhs);
 }

 @Override
 public String toString()
 {
    StringBuilder sb  = new StringBuilder();
    StringBuilder sb2 = new StringBuilder();
    sb.append(getClass().getSimpleName());
    sb.append(" requestID:").append(requestID); // writeOneToString
    sb.append(" requiredReliabilityService:").append(requiredReliabilityService); // writeOneToString
    sb.append(" pad1:").append(pad1); // writeOneToString
    sb.append(" eventType:").append(eventType); // writeOneToString
    sb.append(" time:").append(time); // writeOneToString
    sb.append(" recordIDs: ");
    recordIDs.forEach(r->{ sb2.append(" ").append(r);}); // writeList
    sb.append(sb2.toString().trim());
    // https://stackoverflow.com/questions/2242471/clearing-a-string-buffer-builder-after-loop
    sb2.setLength(0); // reset

   return sb.toString();
 }
} // end of class
